﻿/*
 * Copyright 2009-11 Williams Technologies Limtied.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Kajbity is a trademark of Williams Technologies Limited.
 *
 * http://www.kajabity.com
 */
using System;
using System.Collections;
using System.Xml.Serialization;
using Kajabity.Tools.Forms;

namespace Event_Manager
{
	/// <summary>
	/// Holds all events and maps to the stored XML document.
	/// </summary>
	[XmlRoot("AllEvents")]
	public class EventDocument : Document
	{
		[XmlElement( typeof( ThistleEventMonth ) )]
		public ArrayList Events;
		
		public EventDocument()
		{
		}

		public ThistleEvent GetAt( int index )
		{
			IEnumerator e = Events.GetEnumerator();
			while( e.MoveNext() )
			{
				ThistleEventMonth m = (ThistleEventMonth) e.Current;
				if( index < m.ThistleEvents.Count )
				{
					return (ThistleEvent) m.ThistleEvents[ index ];
				}
				
				index -= m.ThistleEvents.Count;
			}
			
			throw new IndexOutOfRangeException();
		}

		public void RemoveAt( int index )
		{
			IEnumerator e = Events.GetEnumerator();
			while( e.MoveNext() )
			{
				ThistleEventMonth m = (ThistleEventMonth) e.Current;
				if( index < m.ThistleEvents.Count )
				{
					m.ThistleEvents.RemoveAt( index );
					return;
				}
				
				index -= m.ThistleEvents.Count;
			}
			
			throw new IndexOutOfRangeException();
		}
		

		public int Insert( ThistleEvent thistleEvent )
		{
			int index = 0;
			DateTime monthDate = thistleEvent.EventDate.Value;
			monthDate = new DateTime( monthDate.Year, monthDate.Month, 1 );

			for( int month = 0; month < Events.Count; month++ )
			{
				ThistleEventMonth m = (ThistleEventMonth) Events[ month ];
				
				if( monthDate > m.Date )
				{
					m = new ThistleEventMonth( monthDate );
					m.ThistleEvents.Add( thistleEvent );
					Events.Insert( month, m );
					break;
				}
				else if( monthDate == m.Date )
				{
					for( int offset = 0; offset < m.ThistleEvents.Count; offset++ )
					{
						if( thistleEvent.EventDate <= ((ThistleEvent) m.ThistleEvents[ offset ]).EventDate )
						{
							m.ThistleEvents.Insert( offset, thistleEvent );
							index += offset;
							return index;
						}
					}
					
					index += m.ThistleEvents.Add( thistleEvent );
					return index;
				}
				else
				{
					index -= m.ThistleEvents.Count;
				}
			}

			return index;			
		}
	}
}
