﻿/*
 * Copyright 2009-11 Williams Technologies Limtied.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Kajbity is a trademark of Williams Technologies Limited.
 *
 * http://www.kajabity.com
 */
using System;
using System.Diagnostics;
using System.Windows.Forms;
using System.Configuration;

using Google.GData.Calendar;
using Google.GData.Client;
using Google.GData.Extensions;

namespace Event_Manager
{
	/// <summary>
	/// Description of GoogleCalendarUtils.
	/// http://code.google.com/apis/calendar/data/2.0/developers_guide_dotnet.html
	/// </summary>
	public class GoogleCalendarUtils
	{
		// The URL of the calendar.
		private static Uri calendarUri = new Uri( System.Configuration.ConfigurationManager.AppSettings["Calendar.URL"] );
		
		//The application name - could use: Application.ProductName + "-" + Application.ProductVersion;
		private static string appName = "EventManager-0.0.1";
		
		// The user name used to login - usually the email address.
		private static string userName = System.Configuration.ConfigurationManager.AppSettings["Calendar.User.Name"];
		
		// The password used to login to Goolge Calendar.
		private static string password = System.Configuration.ConfigurationManager.AppSettings["Calendar.Password"];
		
		// Alternatively you can use:
		// http://www.google.com/calendar/feeds/calendar@example.com/private-xxxx/basic

		private CalendarService service = null;
		
		public GoogleCalendarUtils()
		{
			// connect to the calendar service
			service = new CalendarService( appName );
			service.setUserCredentials( userName, password );
		}
		
		public void InsertEntry( ThistleEvent thistleEvent )
		{
			EventEntry entry = new EventEntry();
			
//						Debug.WriteLine(
//							"T:" + XmlUtils.ToString( thistleEvent.Title ) + ", " +
//							"S:" + XmlUtils.ToShortDateString( thistleEvent.EventDate ) + " " + thistleEvent.EventTime + ", " +
//							"E:" + XmlUtils.ToShortDateString( thistleEvent.EventDateTo ) + " " + thistleEvent.EventTimeTo + ", " +
//							"L:" + thistleEvent.Location + ", " +
//							"C:" + thistleEvent.Category + ", " +
//							"D:" + XmlUtils.ToString( thistleEvent.Description ) + ", " +
//							"G:" + thistleEvent.Group + ", " +
//							"H:" + thistleEvent.Home + ", " +
//							"P:" + XmlUtils.ToString( thistleEvent.Program ) );

			// Set the title and content of the entry.
			entry.Title.Text = XmlUtils.ToString( thistleEvent.Title );
			
			string html = XmlUtils.ToString( thistleEvent.Description );
			if( html != null && html.Length > 0 )
			{
				entry.Content.Type = "html";
				entry.Content.Content = html;
			}

			// Set a location for the event.
			if( thistleEvent.Location != null && thistleEvent.Location.Length > 0 )
			{
				Where eventLocation = new Where();
				eventLocation.ValueString = thistleEvent.Location;
				entry.Locations.Add( eventLocation );
			}
			
			string text = thistleEvent.EventDate.Value.ToShortDateString() + " " + thistleEvent.EventTime;
			DateTime start = DateTime.Parse( text.Trim() );

			When eventTime = new When();
			eventTime.StartTime = start;
			
			if( thistleEvent.EventTimeTo != null && thistleEvent.EventTimeTo.Length > 0 )
			{
				if( thistleEvent.EventDateTo.HasValue )
				{
					text = thistleEvent.EventDateTo.Value.ToShortDateString() + " " + thistleEvent.EventTimeTo;
				}
				else
				{
					text = thistleEvent.EventDate.Value.ToShortDateString() + " " + thistleEvent.EventTimeTo;
				}
				
				DateTime endTime = DateTime.Parse( text.Trim() );
				eventTime.EndTime = endTime;
			}
			else if( thistleEvent.EventDateTo.HasValue )
			{
				eventTime.EndTime = thistleEvent.EventDateTo.Value;
			}

			entry.Times.Add(eventTime);
			
			if( (thistleEvent.EventTime == null || thistleEvent.EventTime.Length == 0) &&
			   (thistleEvent.EventTimeTo == null || thistleEvent.EventTimeTo.Length == 0) )
			{
				eventTime.AllDay = true;
			}

			AtomPerson author = new AtomPerson( AtomPersonType.Author );
			author.Name = "Simon Williams";
			author.Email = "simon@kajabit.com";
			entry.Authors.Add( author );

			// Send the request and receive the response:
			AtomEntry insertedEntry = service.Insert(calendarUri, entry);
		}
		
		/// <summary>
		/// A static test class to insert a test entry into the calendar.
		/// </summary>
		public static void InsertEntry()
		{
			EventEntry entry = new EventEntry();

			// Set the title and content of the entry.
			entry.Title.Text = "Test Event";
			entry.Content.Content = "This is a test entry - an event.";

			// Set a location for the event.
			Where eventLocation = new Where();
			eventLocation.ValueString = "St James Social Centre, St. James Rd, Orrell, Wigan WN5 7AA";
			entry.Locations.Add(eventLocation);
			When eventTime = new When(DateTime.Now, DateTime.Now.AddHours(2));
			entry.Times.Add(eventTime);

//			AtomPerson author = new AtomPerson( AtomPersonType.Author );
//			author.Name = "Simon Williams";
//			author.Email = "simon@kajabit.com";
//			entry.Authors.Add( author );

			// connect to the calendar service
			CalendarService service = new CalendarService( appName );
			service.setUserCredentials( userName, password );

			// Send the request and receive the response:
			AtomEntry insertedEntry = service.Insert(calendarUri, entry);
		}

		
		/// <summary>
		/// A static method to extract and output some entries from the calendar.
		/// </summary>
		public static void DumpCalendar()
		{
			// create a query object and get 10 entries that start between now & the next 10 days
			EventQuery query = new EventQuery();
			query.Uri = calendarUri;
			query.NumberToRetrieve = 10;
			query.StartTime = System.DateTime.Now;
			query.EndTime = System.DateTime.Today.AddDays(10);

			// connect to the calendar service
			CalendarService service = new CalendarService(appName);
			service.setUserCredentials(userName, password);

			// execute the query & get the results back
			EventFeed calFeed = service.Query(query);

			// iterate through the results
			foreach (EventEntry feedEntry in calFeed.Entries)
			{
				string eventStartDesc = "n/a";
				string eventWhereDesc = "n/a";
				
				if (feedEntry.Times.Count > 0)
				{
					When eventStart = feedEntry.Times[0];
					if (eventStart.AllDay)
						eventStartDesc = "All day";
					else if (eventStart.StartTime.Date == System.DateTime.Today.Date)
						eventStartDesc = "Today at " + eventStart.StartTime.ToShortTimeString();
					else
						eventStartDesc = eventStart.StartTime.ToString();
				}

				if (feedEntry.Locations.Count > 0)
				{
					Where where = feedEntry.Locations[0];
					if (where.ValueString != null || where.Label != null)
						eventWhereDesc = where.Rel + " " + where.Label + " " + where.ValueString;
				}
				
				Debug.WriteLine("Event:");
				Debug.WriteLine("\tTitle: " + feedEntry.Title.Text);
				Debug.WriteLine("\tWhen: " + eventStartDesc);
				Debug.WriteLine("\tWhere: " + eventWhereDesc);
			}
		}
	}
}